@@ -25,14 +25,14 @@ print(ctx)
 runtime = RuntimeEnvBuilder().with_disk_manager_os().with_fair_spill_pool(10000000)
 config = (
     SessionConfig()
-    .with_create_default_catalog_and_schema(True)
+    .with_create_default_catalog_and_schema(enabled=True)
     .with_default_catalog_and_schema("foo", "bar")
     .with_target_partitions(8)
-    .with_information_schema(True)
-    .with_repartition_joins(False)
-    .with_repartition_aggregations(False)
-    .with_repartition_windows(False)
-    .with_parquet_pruning(False)
+    .with_information_schema(enabled=True)
+    .with_repartition_joins(enabled=False)
+    .with_repartition_aggregations(enabled=False)
+    .with_repartition_windows(enabled=False)
+    .with_parquet_pruning(enabled=False)
     .set("datafusion.execution.parquet.pushdown_filters", "true")
 )
 ctx = SessionContext(config, runtime)
