@@ -26,7 +26,9 @@ try:
 except ImportError:
     import importlib_metadata
 
-from . import functions, object_store, substrait
+from datafusion.col import col, column
+
+from . import functions, object_store, substrait, unparser
 
 # The following imports are okay to remain as opaque to the user.
 from ._internal import Config
@@ -45,56 +47,69 @@ from .expr import (
     Expr,
     WindowFrame,
 )
+from .html_formatter import configure_formatter
+from .io import read_avro, read_csv, read_json, read_parquet
 from .plan import ExecutionPlan, LogicalPlan
 from .record_batch import RecordBatch, RecordBatchStream
-from .udf import Accumulator, AggregateUDF, ScalarUDF, WindowUDF
+from .user_defined import (
+    Accumulator,
+    AggregateUDF,
+    ScalarUDF,
+    TableFunction,
+    WindowUDF,
+    udaf,
+    udf,
+    udtf,
+    udwf,
+)
 
 __version__ = importlib_metadata.version(__name__)
 
 __all__ = [
     "Accumulator",
-    "Config",
-    "DataFrame",
-    "SessionContext",
-    "SessionConfig",
-    "SQLOptions",
-    "RuntimeEnvBuilder",
-    "Expr",
-    "ScalarUDF",
-    "WindowFrame",
-    "column",
-    "col",
-    "literal",
-    "lit",
-    "DFSchema",
-    "Catalog",
-    "Database",
-    "Table",
     "AggregateUDF",
-    "WindowUDF",
-    "LogicalPlan",
+    "Catalog",
+    "Config",
+    "DFSchema",
+    "DataFrame",
+    "Database",
     "ExecutionPlan",
+    "Expr",
+    "LogicalPlan",
     "RecordBatch",
     "RecordBatchStream",
+    "RuntimeEnvBuilder",
+    "SQLOptions",
+    "ScalarUDF",
+    "SessionConfig",
+    "SessionContext",
+    "Table",
+    "TableFunction",
+    "WindowFrame",
+    "WindowUDF",
+    "col",
+    "column",
     "common",
+    "configure_formatter",
     "expr",
     "functions",
+    "lit",
+    "literal",
     "object_store",
+    "read_avro",
+    "read_csv",
+    "read_json",
+    "read_parquet",
     "substrait",
+    "udaf",
+    "udf",
+    "udtf",
+    "udwf",
+    "unparser",
 ]
 
 
-def column(value: str):
-    """Create a column expression."""
-    return Expr.column(value)
-
-
-def col(value: str):
-    """Create a column expression."""
-    return Expr.column(value)
-
-
-def literal(value):
+def literal(value) -> Expr:
     """Create a literal expression."""
     return Expr.literal(value)
 
@@ -112,13 +127,6 @@ def str_lit(value):
     return string_literal(value)
 
 
-def lit(value):
+def lit(value) -> Expr:
     """Create a literal expression."""
     return Expr.literal(value)
-
-
-udf = ScalarUDF.udf
-
-udaf = AggregateUDF.udaf
-
-udwf = WindowUDF.udwf
