@@ -26,7 +26,6 @@ from typing import (
     TYPE_CHECKING,
     Any,
     Iterable,
-    List,
     Literal,
     Optional,
     Union,
@@ -38,6 +37,8 @@ try:
 except ImportError:
     from typing_extensions import deprecated  # Python 3.12
 
+from datafusion._internal import DataFrame as DataFrameInternal
+from datafusion.expr import Expr, SortExpr, sort_or_default
 from datafusion.plan import ExecutionPlan, LogicalPlan
 from datafusion.record_batch import RecordBatchStream
 
@@ -49,10 +50,10 @@ if TYPE_CHECKING:
     import polars as pl
     import pyarrow as pa
 
-from enum import Enum
+    from datafusion._internal import DataFrame as DataFrameInternal
+    from datafusion._internal import expr as expr_internal
 
-from datafusion._internal import DataFrame as DataFrameInternal
-from datafusion.expr import Expr, SortExpr, sort_or_default
+from enum import Enum
 
 
 # excerpt from deltalake
@@ -72,7 +73,7 @@ class Compression(Enum):
     LZ4_RAW = "lz4_raw"
 
     @classmethod
-    def from_str(cls, value: str) -> "Compression":
+    def from_str(cls: type[Compression], value: str) -> Compression:
         """Convert a string to a Compression enum value.
 
         Args:
@@ -86,10 +87,13 @@ class Compression(Enum):
         """
         try:
             return cls(value.lower())
-        except ValueError:
-            raise ValueError(
-                f"{value} is not a valid Compression. Valid values are: {[item.value for item in Compression]}"
-            )
+        except ValueError as err:
+            valid_values = str([item.value for item in Compression])
+            error_msg = f"""
+                {value} is not a valid Compression.
+                Valid values are: {valid_values}
+                """
+            raise ValueError(error_msg) from err
 
     def get_default_level(self) -> Optional[int]:
         """Get the default compression level for the compression type.
@@ -103,9 +107,9 @@ class Compression(Enum):
         # https://github.com/apache/datafusion-python/pull/981#discussion_r1904789223
         if self == Compression.GZIP:
             return 6
-        elif self == Compression.BROTLI:
+        if self == Compression.BROTLI:
             return 1
-        elif self == Compression.ZSTD:
+        if self == Compression.ZSTD:
             return 4
         return None
 
@@ -124,7 +128,11 @@ class DataFrame:
         """
         self.df = df
 
-    def __getitem__(self, key: str | List[str]) -> DataFrame:
+    def into_view(self) -> pa.Table:
+        """Convert DataFrame as a ViewTable which can be used in register_table."""
+        return self.df.into_view()
+
+    def __getitem__(self, key: str | list[str]) -> DataFrame:
         """Return a new :py:class`DataFrame` with the specified column or columns.
 
         Args:
@@ -273,14 +281,13 @@ class DataFrame:
 
         def _simplify_expression(
             *exprs: Expr | Iterable[Expr], **named_exprs: Expr
-        ) -> list[Expr]:
+        ) -> list[expr_internal.Expr]:
             expr_list = []
             for expr in exprs:
                 if isinstance(expr, Expr):
                     expr_list.append(expr.expr)
                 elif isinstance(expr, Iterable):
-                    for inner_expr in expr:
-                        expr_list.append(inner_expr.expr)
+                    expr_list.extend(inner_expr.expr for inner_expr in expr)
                 else:
                     raise NotImplementedError
             if named_exprs:
@@ -505,10 +512,15 @@ class DataFrame:
         # This check is to prevent breaking API changes where users prior to
         # DF 43.0.0 would  pass the join_keys as a positional argument instead
         # of a keyword argument.
-        if isinstance(on, tuple) and len(on) == 2:
-            if isinstance(on[0], list) and isinstance(on[1], list):
-                join_keys = on  # type: ignore
-                on = None
+        if (
+            isinstance(on, tuple)
+            and len(on) == 2
+            and isinstance(on[0], list)
+            and isinstance(on[1], list)
+        ):
+            # We know this is safe because we've checked the types
+            join_keys = on  # type: ignore[assignment]
+            on = None
 
         if join_keys is not None:
             warnings.warn(
@@ -521,18 +533,17 @@ class DataFrame:
 
         if on is not None:
             if left_on is not None or right_on is not None:
-                raise ValueError(
-                    "`left_on` or `right_on` should not provided with `on`"
-                )
+                error_msg = "`left_on` or `right_on` should not provided with `on`"
+                raise ValueError(error_msg)
             left_on = on
             right_on = on
         elif left_on is not None or right_on is not None:
             if left_on is None or right_on is None:
-                raise ValueError("`left_on` and `right_on` should both be provided.")
+                error_msg = "`left_on` and `right_on` should both be provided."
+                raise ValueError(error_msg)
         else:
-            raise ValueError(
-                "either `on` or `left_on` and `right_on` should be provided."
-            )
+            error_msg = "either `on` or `left_on` and `right_on` should be provided."
+            raise ValueError(error_msg)
         if isinstance(left_on, str):
             left_on = [left_on]
         if isinstance(right_on, str):
@@ -718,9 +729,11 @@ class DataFrame:
         if isinstance(compression, str):
             compression = Compression.from_str(compression)
 
-        if compression in {Compression.GZIP, Compression.BROTLI, Compression.ZSTD}:
-            if compression_level is None:
-                compression_level = compression.get_default_level()
+        if (
+            compression in {Compression.GZIP, Compression.BROTLI, Compression.ZSTD}
+            and compression_level is None
+        ):
+            compression_level = compression.get_default_level()
 
         self.df.write_parquet(str(path), compression.value, compression_level)
 
@@ -816,7 +829,7 @@ class DataFrame:
         Returns:
             A DataFrame with the columns expanded.
         """
-        columns = [c for c in columns]
+        columns = list(columns)
         return DataFrame(self.df.unnest_columns(columns, preserve_nulls=preserve_nulls))
 
     def __arrow_c_stream__(self, requested_schema: pa.Schema) -> Any:
@@ -856,3 +869,25 @@ class DataFrame:
             DataFrame: After applying func to the original dataframe.
         """
         return func(self, *args)
+
+    def fill_null(self, value: Any, subset: list[str] | None = None) -> DataFrame:
+        """Fill null values in specified columns with a value.
+
+        Args:
+            value: Value to replace nulls with. Will be cast to match column type.
+            subset: Optional list of column names to fill. If None, fills all columns.
+
+        Returns:
+            DataFrame with null values replaced where type casting is possible
+
+        Examples:
+            >>> df = df.fill_null(0)  # Fill all nulls with 0 where possible
+            >>> # Fill nulls in specific string columns
+            >>> df = df.fill_null("missing", subset=["name", "category"])
+
+        Notes:
+            - Only fills nulls in columns where the value can be cast to the column type
+            - For columns where casting fails, the original column is kept unchanged
+            - For columns not in subset, the original column is kept unchanged
+        """
+        return DataFrame(self.df.fill_null(value, subset))
