@@ -18,7 +18,7 @@
 use std::fmt::{self, Display, Formatter};
 
 use datafusion::logical_expr::{logical_plan::Repartition, Expr, Partitioning};
-use pyo3::prelude::*;
+use pyo3::{prelude::*, IntoPyObjectExt};
 
 use crate::{errors::py_type_err, sql::logical::PyLogicalPlan};
 
@@ -121,7 +121,7 @@ impl LogicalNode for PyRepartition {
         vec![PyLogicalPlan::from((*self.repartition.input).clone())]
     }
 
-    fn to_variant(&self, py: Python) -> PyResult<PyObject> {
-        Ok(self.clone().into_py(py))
+    fn to_variant<'py>(&self, py: Python<'py>) -> PyResult<Bound<'py, PyAny>> {
+        self.clone().into_bound_py_any(py)
     }
 }
