@@ -16,7 +16,7 @@
 // under the License.
 
 use datafusion::logical_expr::logical_plan::Unnest;
-use pyo3::prelude::*;
+use pyo3::{prelude::*, IntoPyObjectExt};
 use std::fmt::{self, Display, Formatter};
 
 use crate::common::df_schema::PyDFSchema;
@@ -79,7 +79,7 @@ impl LogicalNode for PyUnnest {
         vec![PyLogicalPlan::from((*self.unnest_.input).clone())]
     }
 
-    fn to_variant(&self, py: Python) -> PyResult<PyObject> {
-        Ok(self.clone().into_py(py))
+    fn to_variant<'py>(&self, py: Python<'py>) -> PyResult<Bound<'py, PyAny>> {
+        self.clone().into_bound_py_any(py)
     }
 }
